@@ -3,7 +3,7 @@ import os
 import random
 from pathlib import Path
 from datetime import datetime
-from typing import Any, Dict, Tuple, List
+from typing import Any, Dict, Tuple, List, Optional
 from threading import Timer
 
 try:
@@ -23,6 +23,7 @@ _fonts = []
 
 MIN_TIMEOUT = 30
 MAX_TIMEOUT = 600
+CODE_LENGTH = 8
 
 digits = "1234567890"
 hexdigits = digits + "ABCDEF"
@@ -104,7 +105,7 @@ class Captcha(types.JsonDeserializable, types.JsonSerializable):
         """
         How many (hex)digits do not match?
         """
-        users_code = self.users_code.ljust(8, "x")
+        users_code = self.users_code.ljust(CODE_LENGTH, "x")
         count = 0
         for i, d in enumerate(users_code):
             if self.correct_code[i] != d: count += 1
@@ -192,7 +193,7 @@ class Captcha(types.JsonDeserializable, types.JsonSerializable):
         if (btn == "BACK"):
             self.users_code = self.users_code[:-1]
         else:
-            self.users_code = (self.users_code + btn)[:8]
+            self.users_code = (self.users_code + btn)[:CODE_LENGTH]
 
         if not self.reply_markup:
             self.reply_markup = callback.message.reply_markup
@@ -213,7 +214,7 @@ class CaptchaManager:
     _handlers = {"on_correct": None, "on_not_correct": None, "on_timeout": None}
     _bot_id = None
 
-    def __init__(self, bot_id: int, default_language: str="en", default_timeout: float=None, fonts: List=None) -> None:
+    def __init__(self, bot_id: int, default_language: str="en", default_timeout: float=None, fonts: List=None, code_length: int=8) -> None:
         """
         The Captcha Manager
 
@@ -226,7 +227,13 @@ class CaptchaManager:
         :param default_language: language to be used if not defined in `send_random_captcha`
         :param default_timeout: timeout to be useed if not defined in `send_random_captcha`
         :param fonts: fonts to be used to generate CAPTCHA images. (.ttf)
+        :param code_length: the lenght of the code. must be between 4-12 chars
         """
+        global CODE_LENGTH
+        if not (4 <= code_length <= 12):
+            raise ValueError("The Code lenghth must be between 4 and 12.")
+        CODE_LENGTH = code_length
+
         self.__class__._bot_id = bot_id
         if default_language.lower() not in languages.keys():
             raise NotImplementedError (f"The Language '{default_language}' is not implemented yet")
@@ -345,20 +352,29 @@ class CaptchaManager:
             return
         
         if btn == "OK":
-            user_id = int(callback.data.split("=")[1])
-            captcha_id = f"{self.__class__._bot_id}={callback.message.chat.id}={user_id}"
-            if (captcha_id in self.captchas):
-                self._check_captcha(self.captchas[captcha_id])
+            if len(captcha.users_code) < CODE_LENGTH:
+                bot.answer_callback_query(callback.id,text=languages[captcha.language]["notfull"], show_alert=True)
+            else:
+                user_id = int(callback.data.split("=")[1])
+                captcha_id = f"{self.__class__._bot_id}={callback.message.chat.id}={user_id}"
+                if (captcha_id in self.captchas):
+                    self._check_captcha(self.captchas[captcha_id])
         else:
             captcha._update(bot, callback)
         
         bot.answer_callback_query(callback.id)
     
     def refresh_captcha(self, bot: TeleBot, captcha: Captcha, 
-            only_digits=False, add_noise=True, timeout: float=None) -> None:
+            only_digits: Optional[bool]=None, add_noise: Optional[bool]=None, timeout: Optional[float]=None) -> None:
+        
         if captcha._timeout_thread is not None:
             captcha._timeout_thread.cancel()
-        timeout = timeout or self.default_timeout
+        
+        # Default settings to previous settings if not set again
+        if only_digits is None: only_digits = captcha._only_digits
+        if add_noise is None: add_noise = captcha._add_noise
+        if timeout is None: timeout = captcha._timeout or self.default_timeout
+
         captcha._refresh(bot, only_digits, add_noise, timeout)
         if timeout:
             if not MIN_TIMEOUT < timeout < MAX_TIMEOUT:
@@ -487,9 +503,21 @@ def _quick_markup(values, row_width=4) -> types.InlineKeyboardMarkup:
     markup.add(*buttons)
     return markup
 
+def _random_code(chars):
+    length = CODE_LENGTH
+    code = ""
+    while length > 0:
+        if length >= len(chars):
+            code += "".join(random.sample(chars, len(chars)))
+            length -= len(chars)
+        else:
+            code += "".join(random.sample(chars, length))
+            length = 0
+    return code
+
 def _random_codeimage(only_digits: bool=False, add_noise: bool=True) -> Tuple:
-    image = ImageCaptcha(256, 128, _fonts, [48, 42, 54])
-    code = "".join(random.sample((digits if only_digits else hexdigits), 8))
+    image = ImageCaptcha(300, 128, _fonts, [48, 42, 54])
+    code = _random_code(digits if only_digits else hexdigits)
     image = image.generate_image(code)
 
     if add_noise:
